/*============================================================================
  File:     InitiateFailover.sql

  Summary:  Initiates the failover - manually.

  Date:     July 2005

  SQL Server Version: 9.00.1187.07 (IDW15)
------------------------------------------------------------------------------
  Copyright (C) 2005 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

:SETVAR PrincipalServer SQLIDW15HA\SQLDev01
:SETVAR MirrorServer SQLIDW15HA\SQLDev02
:SETVAR WitnessServer SQLIDW15HA\SQLExpress
:SETVAR Database2Mirror AdventureWorks
:SETVAR TimeDelay '00:00:01'

---- Failover when in sync
:CONNECT $(PrincipalServer)

-- First, verify current state:
SELECT	db_name(sd.[database_id]) AS [Database Name], 
		sd.mirroring_guid, 
		sd.mirroring_state, 
		sd.mirroring_state_desc,			-- should be SYNCHRONIZED
		sd.mirroring_partner_name, 
		sd.mirroring_witness_name, 
		sd.mirroring_witness_state, 
		sd.mirroring_witness_state_desc, 
		sd.mirroring_role, 
		sd.mirroring_role_desc,				-- should be PRINCIPAL
		sd.mirroring_role_sequence, 
		sd.mirroring_safety_level, 
		sd.mirroring_safety_level_desc, 
		sd.mirroring_safety_sequence, 
		sd.mirroring_failover_lsn 
FROM	sys.database_mirroring AS sd
WHERE	sd.[database_id] = db_id(N'$(Database2Mirror)')

-- Second, force failover when databases are synchronized.
ALTER DATABASE $(Database2Mirror)
	SET PARTNER FAILOVER
go

WAITFOR DELAY $(TimeDelay)
go

-- Third, verify the state after the change - in the Principal:
SELECT	db_name(sd.[database_id]) AS [Database Name], 
		sd.mirroring_guid, 
		sd.mirroring_state, 
		sd.mirroring_state_desc,			-- should be SYNCHRONIZED
		sd.mirroring_partner_name, 
		sd.mirroring_witness_name, 
		sd.mirroring_witness_state, 
		sd.mirroring_witness_state_desc, 
		sd.mirroring_role, 
		sd.mirroring_role_desc,				-- should be MIRROR
		sd.mirroring_role_sequence, 
		sd.mirroring_safety_level, 
		sd.mirroring_safety_level_desc, 
		sd.mirroring_safety_sequence, 
		sd.mirroring_failover_lsn 
FROM	sys.database_mirroring AS sd
WHERE	sd.[database_id] = db_id(N'$(Database2Mirror)')

WAITFOR DELAY $(TimeDelay)
go

-- Forth, verify the state after the change - in the Mirror:
:CONNECT $(MirrorServer)
SELECT	db_name(sd.[database_id]) AS [Database Name], 
		sd.mirroring_guid, 
		sd.mirroring_state, 
		sd.mirroring_state_desc,			-- should be SYNCHRONIZED
		sd.mirroring_partner_name, 
		sd.mirroring_witness_name, 
		sd.mirroring_witness_state, 
		sd.mirroring_witness_state_desc, 
		sd.mirroring_role, 
		sd.mirroring_role_desc,				-- should be MIRROR
		sd.mirroring_role_sequence, 
		sd.mirroring_safety_level, 
		sd.mirroring_safety_level_desc, 
		sd.mirroring_safety_sequence, 
		sd.mirroring_failover_lsn 
FROM	sys.database_mirroring AS sd
WHERE	sd.[database_id] = db_id(N'$(Database2Mirror)')

---- Force failover with potential for data loss
:CONNECT $(PrincipalServer)
ALTER DATABASE $(Database2Mirror)
	SET PARTNER FORCE_SERVICE_ALLOW_DATA_LOSS 
go

-- Flip the servers again - this is hardcoded.
--:CONNECT SQLIDW15HA\SQLDev02
--
--ALTER DATABASE AdventureWorks
--	SET PARTNER FAILOVER
--go