/*============================================================================
  File:     CreditPTwDatabaseSnapshots.sql

  Summary:  This script uses the Partitioned Table database (created on
			drive C:\) to show how snapshots need to be created as well
			as how sparse file population occurs.

  Date:     May 2005

  SQL Server Version: 9.00.1116.08 (April CTP)
------------------------------------------------------------------------------
  Copyright (C) 2005 Kimberly L. Tripp, SYSolutions, Inc.
  All rights reserved.

  For more scripts and sample code, check out 
    http://www.SQLskills.com

  This script is intended only as a supplement to demos and lectures
  given by Kimberly L. Tripp.  
  
  THIS CODE AND INFORMATION ARE PROVIDED "AS IS" WITHOUT WARRANTY OF 
  ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT LIMITED 
  TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND/OR FITNESS FOR A
  PARTICULAR PURPOSE.
============================================================================*/

--drop database creditss1
--drop database creditss2

-- Before this script will work you will need to:
-- (1) Download the zip, unzip and restore the credit database. You
--		can find the zip on the Past Events page from SQLskills.com.
-- (2) Create the multiple filegroup and partitioned table scenario
--		using the script: "CreditPTOnDriveC.sql" 
-- Once CreditPT is created, you can use this script to create a snapshot!
-- NOTE: This script has many comments which are not commented out...
-- In other words, don't try to just "execute" this script, you should
-- work your way through it slowly!

USE CreditPT
go

sp_helpfile
go

CREATE DATABASE [CreditSS1] 
ON  PRIMARY 
( NAME = N'CreditData'
	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1Data.mdfss')
,
( NAME = N'CreditPTChargesStaging'
	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1ChargesStaging.ndfss')
,
( NAME = N'CreditPTFG1File1'
	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PTFG1File1.ndfss')
,
( NAME = N'CreditPTFG2File1'
	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PTFG2File1.ndfss')
,
( NAME = N'CreditPTFG3File1'
	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PTFG3File1.ndfss')
,
( NAME = N'CreditPTFG4File1'
	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS1PTFG4File1.ndfss')
AS SNAPSHOT OF CreditPT
GO

-- with no open trans, runs immediately
-- sizes of files (all SHOW the same - in sys.master_files, etc. but... on disk they're SPARSE)
--CreditData		128k
--CreditLog			n/a
--CreditPTFGFile1	64k
--CreditPTFGFile2	64k
--CreditPTFGFile3	64k
--CreditPTFGFile4	64k
--ChargesStaging	64k

-- change some of august (nicely isolated)
SELECT * 
FROM ChargesPT 
WHERE charge_dt >= '20040806' 
	AND charge_dt < '20040807'
--12992 rows

UPDATE ChargesPT
	SET charge_amt = 20040806
	WHERE charge_dt >= '20040806' 
		AND charge_dt < '20040807'
-- CreditPTFGFile3	768k (copy on write)

SELECT Snap.Charge_no
	, Snap.Charge_amt AS [Snapshot Value]
	, Curr.Charge_amt AS [Current Value]
FROM [CreditSS1].dbo.ChargesPT AS Snap
	JOIN CreditPT.dbo.ChargesPT AS Curr
		ON Snap.charge_no = Curr.charge_no
WHERE Curr.charge_dt >= '20040806' AND Curr.charge_dt < '20040807'
	AND Snap.charge_dt >= '20040806' AND Snap.charge_dt < '20040807'

-- change some of august - but not commit...
SELECT * 
FROM ChargesPT 
WHERE charge_dt >= '20040807' and charge_dt < '20040808'
--12928 rows

BEGIN TRAN
UPDATE ChargesPT
	SET charge_amt = 20040807
	WHERE charge_dt >= '20040807' and charge_dt < '20040808'
-- CreditPTFGFile3	1.31mb
ROLLBACK TRAN
-- CreditPTFGFile3	1.31mb (copy before write, not reverted - yes, it's duplicated...
-- not a big deal - only first change and now the pages are already there)

SELECT Snap.Charge_no, Snap.Charge_amt, Curr.Charge_amt
FROM [CreditSS1].dbo.ChargesPT AS Snap
	JOIN CreditPT.dbo.ChargesPT AS Curr
		ON Snap.charge_no = Curr.charge_no
WHERE Curr.charge_dt >= '20040807' AND Curr.charge_dt < '20040808'
	AND Snap.charge_dt >= '20040807' AND Snap.charge_dt < '20040808'

-- change some of august - but not commit and then go create another snapshot...
select * from ChargesPT 
	where charge_dt >= '20040808' and charge_dt < '20040809'
-- 13760 rows
BEGIN TRAN
update ChargesPT
	SET charge_amt = 20040808
where charge_dt >= '20040808' and charge_dt < '20040809'-- CreditPTFGFile3	1.93mb
-- Create another snapshot while the tran is in flight...
--CREATE DATABASE [CreditSS2] 
--ON  PRIMARY 
--( NAME = N'CreditData'
--	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2Data.mdfss')
--,
--( NAME = N'CreditPTChargesStaging'
--	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2ChargesStaging.ndfss')
--,
--( NAME = N'CreditPTFG1File1'
--	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PTFG1File1.ndfss')
--,
--( NAME = N'CreditPTFG2File1'
--	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PTFG2File1.ndfss')
--,
--( NAME = N'CreditPTFG3File1'
--	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PTFG3File1.ndfss')
--,
--( NAME = N'CreditPTFG4File1'
--	, FILENAME = N'c:\Program Files\Microsoft SQL Server\MSSQL.1\MSSQL\Data\CreditSS2PTFG4File1.ndfss')
--AS SNAPSHOT OF CreditPT
--GO

-- You Will need to go to another window to do it as it cannot be rolled back!
-- Review file sizes...
--CreditData		512k
--CreditLog			n/a
--CreditPTFGFile1	64k
--CreditPTFGFile2	64k
--CreditPTFGFile3	768k -- this is due to the pending transaction
--CreditPTFGFile4	64k
--ChargesStaging	64k
ROLLBACK TRAN
--CreditData		512k
--CreditLog			n/a
--CreditPTFGFile1	64k
--CreditPTFGFile2	64k
--CreditPTFGFile3	768k
--CreditPTFGFile4	64k
--ChargesStaging	64k

-- Now that we have two snapshots - what happens with a modification?
-- change some of august
SELECT * 
FROM ChargesPT 
WHERE charge_dt >= '20040914' AND charge_dt < '20040915'
--13344 rows

UPDATE ChargesPT
	SET charge_amt = 20040914
	WHERE charge_dt >= '20040914' and charge_dt < '20040915'
--SS1 - CreditPTFGFile4	704k
--SS2 - CredtiPTFGFile4	704k

select * from sys.master_files where database_id = db_id('credit')
select * from sys.master_files where database_id = db_id('creditSS1')
select * from sys.master_files where database_id = db_id('creditSS2')

-- And a more complete query is this (make sure to change the database name):
SELECT db_name(v.database_id) AS [Database Name]
	, v.file_id AS [File ID]
	, CASE WHEN is_sparse = 0 THEN 'No' ELSE 'Yes' END AS [Sparse File]
	, f.[name] AS [File Name]
	, size_on_disk_bytes/1024 AS [Size (KB)]
	, f.physical_name AS [Physical File Name] 
FROM sys.dm_io_virtual_file_stats (db_id('CreditSS1'), -1) AS v
	JOIN sys.master_files AS f ON v.file_id = f.file_id
WHERE f.database_id = v.database_id AND f.database_id = db_id('CreditSS1')

-- revert back to unmodified version...
USE master
go

RESTORE DATABASE CreditPT
	FROM database_snapshot = 'creditss1'
--Msg 3137, Level 16, State 4, Line 1
--Database can not be reverted. Either the primary 
-- or the snapshot names are improperly specified, all other snapshots have not 
--been dropped, or there are missing files.
--Msg 3013, Level 16, State 1, Line 1
--RESTORE DATABASE is terminating abnormally.

DROP DATABASE creditss2
-- fast... :)

RESTORE DATABASE credit 
FROM database_snapshot = 'creditss1'
-- works!

-- once you revert the image is restored AND the snapshot STAYS
SS1 - CreditPTFGFile3	1.93mb

-- change some of august AGAIN
SELECT * 
FROM ChargesPT 
WHERE charge_dt >= '20040806' and charge_dt < '20040807'
--12992 rows
-- File sizes don't change. Snapshot already has these values...

-- change some of july that was never changed...
SELECT * 
FROM ChargesPT 
WHERE charge_dt >= '20040716' and charge_dt < '20040717'
--13424 rows

UPDATE ChargesPT
	SET charge_amt = 20040716
	WHERE charge_dt >= '20040716' and charge_dt < '20040717'
SS1 - CreditPTFGFile3	2.56mb unchanged...
-- 768KB for July Data

-- can we revert back...again?!
RESTORE DATABASE CreditPT 
	FROM database_snapshot = 'creditss1'
-- works!

SELECT Snap.Charge_no
	, Snap.Charge_amt AS [Snapshot Value]
	, Curr.Charge_amt AS [Current Value]
FROM [CreditSS1].dbo.ChargesPT AS Snap
	JOIN CreditPT.dbo.ChargesPT AS Curr
		ON Snap.charge_no = Curr.charge_no
WHERE Curr.charge_dt >= '20040806' AND Curr.charge_dt < '20040807'
	AND Snap.charge_dt >= '20040806' AND Snap.charge_dt < '20040807'
